@@ -142,32 +142,11 @@ double ccaSplitAbsoluteDifference (unsigned int n,
                                    double     **feature,
                                    unsigned int featureCount);
 
-double ccaSplitWeightedSum (unsigned int n,
-                            char        *membership,
-                            double      *time,
-                            double      *event,
-
-                            unsigned int eventTypeSize,
-                            unsigned int eventTimeSize,
-                            double      *eventTime,
-
-                            double      *response,
-                            double       mean,
-                            double       variance,
-                            unsigned int maxLevel,
-
-                            double     **feature,
-                            unsigned int featureCount);
-
 unsigned int *alloc_uivector(unsigned int nh);
 void          dealloc_uivector(unsigned int *v, unsigned int nh);
 
-double       *alloc_dvector(double *v, unsigned int nh);
-void          dealloc_dvector(double *v, unsigned int nh);
-
 unsigned int **alloc_uimatrix(unsigned int n2h, unsigned int nh);
 void          dealloc_uimatrix(unsigned int **v, unsigned int n2h, unsigned int nh);
 
-double *alloc_dvector2(unsigned int ncols);
-void dealloc_dvector2(double *v, unsigned int ncols);
-
+double *alloc_dvector(unsigned int nh);
+void dealloc_dvector(double *v);
